/* 
   PkgsSel 
   Copyright (C) 1999 Jarek Woloszyn (yossa@pld.org.pl)
   $Id: pkgssel.c,v 1.8 2001/09/20 19:38:15 pawelk Exp $
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
 */

#define FNAME_SAVE "/tmp/rpm.list"
#define OUTPUTFILE "install.stdout"

#include <stdlib.h>
#include <string.h>
#include <newt.h>
#include <trurl/trurl.h>
#include <rpmpldi.h>
#include <newt-addon.h>

#include "pkgssel-data.h"
#include "pkgssel-show.h"
#include "regexp-search.h"
#include "utils.h"
#include "pkgssel.h"
#include "wuch.h"

unsigned LISTWIDTH = 32;
unsigned MAINWINWIDTH = 78;
unsigned MAINWINHEIGHT = 19;
int mode;

void handleBlackList(tn_list *bp){
    struct newtExitStruct answer;
	tn_list * chain;
	int i,j;
	TPkgInfo *pk;
	if(bp && n_list_size(bp)){
		char text[50000]="", buf[10000];
		newtComponent f, textBox, bLeave, bRemove, bBreak;
		
		newtCenteredWindow(60,20,"Black list problems");
		f = newtForm(NULL, NULL, 0);
		for(i=0; i < n_list_size(bp); i++){
			chain=(tn_list *)n_list_nth(bp,i);
			pk = (TPkgInfo*)n_list_nth(chain,0);
			sprintf(buf , "Package '%s' from black list is needed by:\n",
					pk->name);
			strcat(text, buf);
			for(j=1; j< n_list_size(chain) ; j++){
				pk = (TPkgInfo*)n_list_nth(chain,j);
				strcat(text, "  ");
				strcat(text, pk->name);
				strcat(text, "\n");
			}
			strcat(text,"\n");
		}
		strcat(text,"\n'Leave' -- install all needed packages, even from balck list\n'Remove' -- dont install any package listed above\n'Break' -- don't install packages from black list, install all other");
		textBox = newtTextbox(2,2,55,12,
				NEWT_TEXTBOX_WRAP|NEWT_TEXTBOX_SCROLL);
		newtTextboxSetText(textBox,text);
		bLeave  = newtButton(2,16,"Leave");
		bRemove = newtButton(14,16,"Remove");
		bBreak  = newtButton(26,16,"Break");
		newtFormAddComponents(f,textBox,bLeave,bRemove, bBreak,NULL);
		newtFormRun(f, &answer);
		if(answer.u.co == bLeave){
					log_message("%s -- bLeave",__FUNCTION__);
		}
		if(answer.u.co == bRemove){
			log_message("%s -- bRemove",__FUNCTION__);
			for(i=0; i<n_list_size(bp); i++){
				chain=(tn_list *)n_list_nth(bp, i);
				for(j=0;j< n_list_size(chain); j++){
					pk=(TPkgInfo *) n_list_nth(chain,j);
					pk->selected =0;
				}
			}
		}
		if(answer.u.co == bBreak){
			log_message("%s -- bBreak",__FUNCTION__);
			for(i=0;i <n_list_size(bp); i++){
				chain=(tn_list *)n_list_nth(bp,i);
				pk=(TPkgInfo *)n_list_nth(chain,0);
				pk->selected=0;
			}
		}
		newtPopWindow();
	}
}


/* Fills the main list with data from group array
 * if subdir!=0 then creates up (..) directory entry
 */
static void lbRefill(newtComponent lb, tn_array * group, int subdir)
{
    int i, p;
    char tmpbuf[256];
    char vis_flag, black;
    TSelectorItem *item;
    TPkgSet *pkset;
    
    newtChecklistClear(lb);
    p = 0;
    pkset=rpmpldi_getPkgsSet();
    if (subdir) {
	newtChecklistAppendEntry(lb, "..", 0);
	p++;
    }
    for (i = 0; i < n_array_size(group); i++) {
	item = (TSelectorItem *) n_array_nth(group, i);
	/*  create the list */
	if ((item->type & SEL_PACKET) == SEL_PACKET) {

		vis_flag='N';
		if(pkgs_array_nth(pkset,item->pkgnum)->flags & TPKGINF_EQUAL)
			vis_flag='i';
		if(pkgs_array_nth(pkset,item->pkgnum)->flags & TPKGINF_UPGRADE)
			vis_flag='U';
		if(pkgs_array_nth(pkset,item->pkgnum)->flags & TPKGINF_OLDER)
			vis_flag='O';
		if(pkgs_array_nth(pkset,item->pkgnum)->flags &
				TPKGINF_DIFF_RELEASE)
			vis_flag=tolower(vis_flag);
		if(pkgs_array_nth(pkset,item->pkgnum)->flags & TPKGINF_BLACK)
			black='B';
		else
			black=' ';
						
	    sprintf(tmpbuf, "%c%c %-28.28s", black, vis_flag, item->nameptr);
	    newtChecklistAppendCheckbox(lb, tmpbuf,
					*item->selptr, (void *) item);
	    newtChecklistSetCheckboxPtr(lb, p, item->selptr);
	    p++;
	} else if ((item->type & SEL_DIR) == SEL_DIR) {
	    strcpy(tmpbuf, item->nameptr);
	    strcat(tmpbuf, "/");
	    newtChecklistAppendEntry(lb, tmpbuf, (void *) item);
	    p++;
	}
    }
}

/* Shows description of the packet */
static void show_info(TSelectorItem * item)
{
    newtComponent all, btOK;	/* ui widgets */
    newtComponent txtInfo;
    char *btTxt = _("OK");	/* we do not use grids */
    char *txt;

    /* ui stuff */
    newtOpenWindow(1, 3, MAINWINWIDTH, MAINWINHEIGHT, item->nameptr);

    all = newtForm(NULL, NULL, 0);
    btOK = newtCompactButton(MAINWINWIDTH - 4 - strlen(btTxt), MAINWINHEIGHT - 1, btTxt);
    newtPushHelpLine(_("Enter/Esc - Exit"));

    txtInfo = newtTextbox(1, 1, MAINWINWIDTH - 3, MAINWINHEIGHT - 3, NEWT_FLAG_WRAP |
			  NEWT_FLAG_SCROLL);
    txt = (char *) rpmpldi_header_entry_get(item->pkgnum,
					    RPMTAG_DESCRIPTION);
    newtTextboxSetText(txtInfo, (txt ? txt : _("No Description")));
    newtFormAddComponents(all, txtInfo, btOK, NULL);
    newtFormAddHotKey(all, 27);
    newtRunForm(all);
    newtFormDestroy(all);
    newtPopWindow();
    newtPopHelpLine();
}

/* Shows install.stdout */
static void show_output()
{
    newtComponent all, btOK;	/* ui widgets */
    newtComponent txtInfo;
    char *btTxt = _("OK");	
    char *txt;
    char buf[100];
    FILE *f;
    long len;

    statusMessage(NULL,_("Loading install.stdout file"));
    if ((f=fopen(OUTPUTFILE,"rb"))==NULL) {
      txt = strdup(_("Cannot open file install.stdout"));
    }
    else {
      fseek(f,0,SEEK_END);
      len=ftell(f);
      txt=(char *)malloc(len*sizeof(char)+5);
      if (!txt) {
	  sprintf(buf,_("Cannot allocate %li bytes"),len);
	  txt=strdup(buf);
      }
      else {
         fseek(f,0,SEEK_SET);
	 fread(txt,len,1,f);
	 txt[len]=0;
      }
      fclose(f);
    }
    txtInfo = newtTextbox(1, 1, MAINWINWIDTH - 3, MAINWINHEIGHT - 3, NEWT_FLAG_WRAP |
			  NEWT_FLAG_SCROLL);
    newtTextboxSetText(txtInfo, (txt ? txt : _("Empty")));
    
    destroy_statusMessage();

    /* ui stuff */
    newtOpenWindow(1, 3, MAINWINWIDTH, MAINWINHEIGHT, OUTPUTFILE);

    all = newtForm(NULL, NULL, 0);
    btOK = newtCompactButton(MAINWINWIDTH - 4 - strlen(btTxt), MAINWINHEIGHT - 1, btTxt);
    newtPushHelpLine(_("Enter/Esc - Exit"));

    newtFormAddComponents(all, txtInfo, btOK, NULL);
    newtFormAddHotKey(all, 27);
    newtRunForm(all);
    newtFormDestroy(all);
    newtPopWindow();
    newtPopHelpLine();
    if (txt) free(txt);
}

/* Main selector function */
/* Big procedure which looks messy - all because of the newt formexit code */
static int selector(void)
{
    int rc = PKGSSEL_INSTALL,  subdir_num = 0; /* number of subdirs */
    int wasf10 = 0;                            /* user pressed f10 */
    int actualDeps = 0;			      
    newtComponent all, btOK, btQuit, btShowOutput;	/* ui widgets */
    newtComponent lbGrps, lblGrpName, lblSummary;
    newtComponent btAddDeps, btRemoveDeps, btSave, btReread;
    newtComponent lbls[20];	/* right panel */
    TSelectorItem *item;	/* tmp selector item */
    tn_array *actDir;		/* directory we are in */
    tn_array *subDirs;		/* array which keeps history */
    char subdir_name[256] = "/";	/* directory path */
    char tmpbuf[256], *tmpptr=NULL;
    char *search = NULL;	/* old search pattern */
    int newtHotKeys[11] =
    {NEWT_KEY_F10, NEWT_KEY_F1, NEWT_KEY_F3, 'u', 'U', 't', 'T',
     '*', '+', '-', 0};
    static int size_type = SIZE_KB;	/* type of the size (kb, Mb) */
    int i, selected, *tmpint;	/* tmp */
    struct newtExitStruct answer;
    newtGrid grid, buttons, middle, labels;

    /* ui stuff - allocate all components */
    newtPushHelpLine(_("F1,F3,Enter - Info, Space,+,-,* - Select, "
		       "u - Go up, "
		       "t - Size type, F10 - exit"));

    /*  checkbox list */
    lbGrps = newtChecklist(-1, -1, MAINWINHEIGHT - 6, NEWT_FLAG_SCROLL |
			   NEWT_FLAG_RETURNEXIT);
    newtChecklistSetWidth(lbGrps, LISTWIDTH);
    newtChecklistSetMCType(lbGrps, 1);

    /*  main form */
    all = newtForm(NULL, NULL, 0);

    /*  labels */
    lblGrpName = newtLabel(-1, -1, subdir_name);
    /*  this label will set window windth, so make it big */
    lblSummary = newtLabel(-1, -1, pkgsMkEmptyString('-', MAINWINWIDTH - 2));

    labels = newtCreateGrid(1, MAINWINHEIGHT - 6);
    for (i = 0; i < 13; i++) {
	lbls[i] = newtLabel(-1, -1, "");
	newtGridSetField(labels, 0, i, NEWT_GRID_COMPONENT, lbls[i],
			 0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
	newtFormAddComponent(all, lbls[i]);
    }

    middle = newtCreateGrid(2, 1);
    newtGridSetField(middle, 1, 0, NEWT_GRID_SUBGRID, labels,
		     1, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
    newtGridSetField(middle, 0, 0, NEWT_GRID_COMPONENT, lbGrps,
		     0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);

	if(mode == MODE_LIST)
	    buttons = newtCompactButtonBar(_("Reread db"),&btReread,
			    		 _("Add Dep*s"), &btAddDeps,
					 _("Rem Dep*s"), &btRemoveDeps,
					 _("Save List"), &btOK,
					 _("Output"), &btShowOutput,
					 _("Quit"), &btQuit,
					   NULL);
	else 
		buttons = newtCompactButtonBar(_("Reread db"),&btReread,
			    		 _("Add Dep*s"), &btAddDeps,
					 _("Rem Dep*s"), &btRemoveDeps,
					 _("Save list"),&btSave,
					 _("Run"), &btOK,
					 _("Output"), &btShowOutput,
					 _("Quit"), &btQuit,
					   NULL);

		
    grid = newtCreateGrid(1, 4);
    newtGridSetField(grid, 0, 0, NEWT_GRID_COMPONENT, lblGrpName,
		     0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
    newtGridSetField(grid, 0, 1, NEWT_GRID_SUBGRID, middle,
		     0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
    newtGridSetField(grid, 0, 2, NEWT_GRID_COMPONENT, lblSummary,
		     0, 0, 0, 1, NEWT_ANCHOR_LEFT, 0);
    newtGridSetField(grid, 0, 3, NEWT_GRID_SUBGRID, buttons,
		     0, 0, 0, 0, NEWT_ANCHOR_RIGHT, 0);

    newtGridAddComponentsToForm(grid, all, 1);
    /*  show window */
    newtGridWrappedWindowAt(grid, _("Packages Panel"), 1, 3);
    newtGridFree(grid, 1);

    /*  set information labels */
    newtLabelSetText(lbls[0], _("Package:"));
    newtLabelSetText(lbls[1], _("  Currently in ver:"));
    newtLabelSetText(lbls[2], _("  Size:"));
    newtLabelSetText(lbls[3], _("Current Directory:"));
    newtLabelSetText(lbls[8], _("All Packages:"));

    for (i = 0; newtHotKeys[i]; i++)
	newtFormAddHotKey(all, newtHotKeys[i]);

    /*  create history array of dirs */
    subDirs = n_array_new(30, NULL, NULL);
    actDir = selGroups;

    /*  Fill the list */
    lbRefill(lbGrps, actDir, subdir_num);
    /*  and right panel  */
    pkgsCountDir(actDir, &lbls[4], size_type);
    pkgsCountAllDirs(&lbls[9], size_type);
    newtRefresh();
    
    while (1) {
	newtFormRun(all, &answer);
	/* Handle hot keys */
	if (answer.reason == NEWT_EXIT_HOTKEY) {
	    selected = 0;
	    switch (answer.u.key) {
		/*  up directory */
	    case 'u':
	    case 'U':
		if (n_array_size(subDirs) > 0) {
		    newtChecklistSetCurrent(lbGrps, 0);
		    /* checkbox part will handle it */
		    answer.u.co = lbGrps;
		    newtChecklistSetStatus(lbGrps, NEWT_CHECKLIST_ENTER);
		}
		break;
		/*  type of data in right panel       */
	    case 't':
	    case 'T':
		size_type++;
		size_type %= 3;
		pkgsCountDir(actDir, &lbls[4], size_type);
		pkgsCountAllDirs(&lbls[9], size_type);
		answer.u.co = lbGrps;
		newtChecklistSetStatus(lbGrps, NEWT_CHECKLIST_MOVE);
		break;
		/*  Package selection         */
	    case '+':
		selected = 3;
		tmpptr = winEntryDialogTwoCompactBtn(_("Select"), "", 60,
					      (search ? search : "*"), 0);
	    case '-':
		if (!selected) {
		    selected = 2;
		    tmpptr = winEntryDialogTwoCompactBtn(_("UnSelect"), "", 60,
					     (search ? search : "*"), 0);
		}
		if (!tmpptr)
		    break;
		if (search)
		    free(search);
		search = tmpptr;
	    case '*':
		if (!selected)
		    selected = 1;

		/*  set falags for appropriate pkgs */
		for (i = 0; i < n_array_size(actDir); i++) {
		    item = n_array_nth(actDir, i);
		    if ((item->type & SEL_PACKET) == SEL_PACKET) {
			if (selected - 2 < 0)
			    *item->selptr = 1 - *item->selptr;
			else if (!regexp_search(search, item->nameptr))
			    *item->selptr = selected - 2;

			newtChecklistSetValueByKey(lbGrps, item,
						   *item->selptr);
		    }
		}
		pkgsCountDir(actDir, &lbls[4], size_type);
		pkgsCountAllDirs(&lbls[9], size_type);
		break;
	    case NEWT_KEY_F1:
	    case NEWT_KEY_F3:
		item = (TSelectorItem *) newtChecklistGetCurrent(lbGrps);
		if (item && (item->type & SEL_PACKET) == SEL_PACKET)
		    show_info(item);
                break;	

            case NEWT_KEY_F10:
		wasf10 = 1;
		break;
	    }
	}
	if ((wasf10 || answer.u.co == btQuit) && 
	    questionMessage("?", _("Do you really want to "
                                   "abort installation?"))) 
                    rc = PKGSSEL_EXIT;
	wasf10 = 0;

	if(answer.u.co == btShowOutput)
	    show_output();
	if(answer.u.co == btSave){
		FILE *f;
		TPkgSet *pkgs;
		TPkgInfo *pkg;
		pkgs=rpmpldi_getPkgsSet();
		f=fopen(FNAME_SAVE,"w");
		if(f==NULL){
			log_message(_("can't open file %s for write"),FNAME_SAVE);
			newtWinMessage(_("Error"),_("OK"),
					_("Can't open file for write")
					);
		}else{
			for(i=0;i<pkgs_array_num(pkgs);i++){
				pkg=pkgs_array_nth(pkgs,i);
				if(pkg->selected){
					if(pkg->flags&(TPKGINF_UPGRADE|
						TPKGINF_EQUAL|					                                TPKGINF_OLDER)){
						fprintf(f,"U ");
					}else{
						fprintf(f,"i ");
					}
					fprintf(f,"%s\n",pkg->fname);
				}
			}
			fclose(f);	
			newtWinMessage(_("Information"),_("OK"),
				_("List of selected packages saved in file: ")
				FNAME_SAVE);
		}
	}
			
	/*  Deps buttons */
	if (answer.u.co == btRemoveDeps || answer.u.co == btAddDeps) {
	    rpmpldi_remove_deps();
	    actualDeps = 0;
	    if (answer.u.co == btAddDeps) {
			tn_list * bp;
			rpmpldi_add_deps();
	        actualDeps = 1;
			
			bp = rpmpldi_check_black();
			handleBlackList(bp);
			rpmpldi_freeBlack(bp);
		
		
	    }
	    /*  reread current directory */
	    if (subdir_num) {
		item = (TSelectorItem *) n_array_pop(subDirs);
		n_array_push(subDirs, (void *) item);
		if ((item->type & SEL_ACTIVE) == SEL_ACTIVE) {
		    n_array_clean(actDir);
		    pkgsCreateActive(actDir);
		    item = NULL;
		} else
		    item = (TSelectorItem *) newtChecklistGetCurrent(lbGrps);
		lbRefill(lbGrps, actDir, subdir_num);
		newtChecklistSetCurrentByKey(lbGrps, item);
	    }
	    /* same as NEWT_CHECKLIST_SELECT */
	    pkgsCountDir(actDir, &lbls[4], size_type);
	    pkgsCountAllDirs(&lbls[9], size_type);
	    /* same as NEWT_CHECKLIST_MOVE */
	    answer.u.co = lbGrps;
	    newtChecklistSetStatus(lbGrps, NEWT_CHECKLIST_MOVE);
	}
	/* We treat this widget in different way, because the form returns
	 * many more keys than normal newt widgets */
	if (answer.u.co == lbGrps) {
	    switch (newtChecklistGetStatus(lbGrps)) {
		/* if user pressed Enter */
	    case NEWT_CHECKLIST_ENTER:
		item = (TSelectorItem *) newtChecklistGetCurrent(lbGrps);
		/* handle up (..) directory */
		if (item == NULL && subdir_num) {
		    /* pop addresses from history array */
		    item = (TSelectorItem *) n_array_pop(subDirs);
		    actDir = (tn_array *) n_array_pop(subDirs);
		    /*  it is a dynamic directory, so we have to clean it */
		    if ((item->type & SEL_ACTIVE) == SEL_ACTIVE)
			n_array_clean(item->dir);
		    tmpptr = pkgsChangeDir(subdir_name, item->nameptr, -1);
		    newtLabelSetText(lblGrpName, tmpptr);
		    free(tmpptr);
		    subdir_num--;
		    lbRefill(lbGrps, actDir, subdir_num);
		    newtChecklistSetCurrentByKey(lbGrps, item);
		    pkgsCountDir(actDir, &lbls[4], size_type);
		    /* handle directories */
		} else if ((item->type & SEL_DIR) == SEL_DIR) {
		    /* put current dir on the stack */
		    n_array_push(subDirs, (void *) actDir);
		    n_array_push(subDirs, (void *) item);
		    actDir = item->dir;
		    tmpptr = pkgsChangeDir(subdir_name, item->nameptr, 1);
		    newtLabelSetText(lblGrpName, tmpptr);
		    free(tmpptr);
		    subdir_num++;
		    /*  create dynamic directory */
		    if ((item->type & SEL_ACTIVE) == SEL_ACTIVE) {
			pkgsCreateActive(actDir);
		    }
		    lbRefill(lbGrps, actDir, subdir_num);
		    pkgsCountDir(actDir, &lbls[4], size_type);
		} else if ((item->type & SEL_PACKET) == SEL_PACKET)
		    show_info(item);
		break;
		/* now play with all cursors, pgup, pgdown, etc */
	    case NEWT_CHECKLIST_SELECT:
		actualDeps = 0;
		pkgsCountDir(actDir, &lbls[4], size_type);
		pkgsCountAllDirs(&lbls[9], size_type);
		/*  change datas in right panel & summary */
	    case NEWT_CHECKLIST_MOVE:
		item = (TSelectorItem *) newtChecklistGetCurrent(lbGrps);
		if (item && (item->type & SEL_PACKET) == SEL_PACKET) {
                    TPkgSet *pkgs;
                    TPkgInfo *pkg;
                    pkgs=rpmpldi_getPkgsSet();
                    pkg=pkgs_array_nth(pkgs,item->pkgnum);
                    if (item->epoch)
	  	       sprintf(tmpbuf, _("Package: %s %i:%s-%s"), 
			    item->nameptr,
			    *item->epoch,
			    item->ver, item->release);
		    else
	  	       sprintf(tmpbuf, _("Package: %s %s-%s"), 
			    item->nameptr,
			    item->ver, item->release);
	    
		    newtLabelSetText(lbls[0], tmpbuf);
		    /* show packet summary */
		    tmpptr = (char *) rpmpldi_header_entry_get(item->pkgnum,
							 RPMTAG_SUMMARY);
		    /* cut when too long */
		    if (tmpptr) {
			strncpy(tmpbuf, tmpptr, 250);
			strcpy(tmpbuf + MAINWINWIDTH - 5, "...");
			newtLabelSetText(lblSummary, tmpbuf);
		    } else
			newtLabelSetText(lblSummary, _("(none)"));
		    rpmpldi_header_entry_free(tmpptr);
		    tmpint = (int *) rpmpldi_header_entry_get(item->pkgnum,
							    RPMTAG_SIZE);
		    sprintf(tmpbuf, _("  Currently in ver: %s"), 
                                     (pkg->installedEVR ? 
                                      pkg->installedEVR : "") );
		    newtLabelSetText(lbls[1], tmpbuf);
			         
		    pkgsSetLabel(lbls[2], _("  Size: "),
				 (tmpint ? *tmpint : 0), size_type);
		    rpmpldi_header_entry_free(tmpint);
		} else {
		    newtLabelSetText(lbls[0], _("Package: "));
		    newtLabelSetText(lbls[2], _("  Size: "));
		    newtLabelSetText(lblSummary,
			       pkgsMkEmptyString('-', MAINWINWIDTH - 2));
		}
	    }
	}
	if(answer.u.co == btReread)
		rc=PKGSSEL_REREAD_DB;
	if (answer.u.co == btOK &&
	   (actualDeps || questionMessage(_("?"), 
			       _("Dependencies are not up-to-date. "
                                 "You should Add Dependencies "
				 "before installing. "
				 "Do you want to continue?")))) {
	    rc=PKGSSEL_INSTALL;
	    break;
	}
	
	if (answer.u.co==btReread || !rc)
	    break;
	newtRefresh();
    };

    if (search)
	free(search);
    newtPopHelpLine();
    newtPopWindow();
    return rc;
}

int pkgsSelector_init(void)
{
    statusMessage(NULL, _("Loading data structures..."));
    if (!pkgsLoadGroups()) {
	destroy_statusMessage();
	return 0;
    }
    destroy_statusMessage();
    return 1;
}

#define max(x,y) ((x)>(y) ? (x) : (y))

int pkgsSelector(int m)
{
    int x,y;
    newtGetScreenSize(&x,&y);
    MAINWINHEIGHT = max(19,y-6); 
    MAINWINWIDTH = max(78,x-2); 
    LISTWIDTH = max(32,x-48);
	mode = m;
    return selector();
}

void pkgsSelector_finish(void)
{
    pkgsFreeGroups();
}

/* vim: set ts=4 sw=4: */
